package me.hammerle.snuviengine.api;

import java.nio.FloatBuffer;
import org.lwjgl.BufferUtils;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL15.*;
import static org.lwjgl.opengl.GL20.*;
import static org.lwjgl.opengl.GL30.*;

public class Chunk
{
    private final int vao;
    private final int vbo;
    private boolean dirty = true;
    private final FloatBuffer buffer = BufferUtils.createFloatBuffer(Map.PARTION * Map.PARTION * 24);
    
    protected Chunk()
    {
        vao = glGenVertexArrays();
        vbo = glGenBuffers();
        
        glBindVertexArray(vao);
        glBindBuffer(GL_ARRAY_BUFFER, vbo);
        
        glEnableVertexAttribArray(0);
        glVertexAttribPointer(0, 2, GL_FLOAT, false, 16, 0);
        
        glEnableVertexAttribArray(1);  
        glVertexAttribPointer(1, 2, GL_FLOAT, false, 16, 8);
    }
    
    protected void delete()
    {
        glDeleteVertexArrays(vao);
        glDeleteBuffers(vbo);
    }
    
    private void build(short[][] data, int sx, int sy, TextureAtlas atlas)
    {
        dirty = false;
        float f = Engine.TILE_SIZE * Engine.SCALE;
        float t = 1.0f / 2048.0f;
        
        for(int x = sx; x < sx + Map.PARTION; x++)
        {
            for(int y = sy; y < sy + Map.PARTION; y++)
            {
                if(data[x][y] == -1)
                {
                    for(int i = 0; i < 24; i++)
                    {
                        buffer.put(-1.0f);
                    }
                }
                else
                {
                    float minX = atlas.getTextureMinX(data[x][y], false) + t;
                    float minY = atlas.getTextureMinY(data[x][y], false) + t;
                    float maxX = atlas.getTextureMaxX(data[x][y], false) - t;
                    float maxY = atlas.getTextureMaxY(data[x][y], false) - t;

                    buffer.put(x * f);
                    buffer.put((y + 1) * f);

                    buffer.put(minX);
                    buffer.put(maxY);

                    buffer.put(x * f);
                    buffer.put(y * f);

                    buffer.put(minX);
                    buffer.put(minY);

                    buffer.put((x + 1) * f);
                    buffer.put((y + 1) * f);

                    buffer.put(maxX);
                    buffer.put(maxY);

                    buffer.put((x + 1) * f);
                    buffer.put((y + 1) * f);

                    buffer.put(maxX);
                    buffer.put(maxY);

                    buffer.put(x * f);
                    buffer.put(y * f);

                    buffer.put(minX);
                    buffer.put(minY);

                    buffer.put((x + 1) * f);
                    buffer.put(y * f);

                    buffer.put(maxX);
                    buffer.put(minY);
                }
            }
        }
        
        buffer.flip();
        
        glBufferData(GL_ARRAY_BUFFER, buffer, GL_STATIC_DRAW);
    }
    
    protected void draw(short[][] data, int sx, int sy, TextureAtlas atlas)
    {
        glBindVertexArray(vao);
        glBindBuffer(GL_ARRAY_BUFFER, vbo);
        
        if(dirty)
        {
            build(data, sx, sy, atlas);            
        }
        
        glDrawArrays(GL_TRIANGLES, 0, Map.PARTION * Map.PARTION * 6);
    }
}
