package me.hammerle.snuviengine.api;

import java.nio.FloatBuffer;
import org.lwjgl.BufferUtils;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL15.*;
import static org.lwjgl.opengl.GL20.*;
import static org.lwjgl.opengl.GL30.*;

public class ColorRenderer
{
    private int vao;
    private int vbo;
    
    private float r = 0.0f;
    private float g = 0.0f;
    private float b = 0.0f;
    private float a = 0.0f;
    
    private FloatBuffer buffer = BufferUtils.createFloatBuffer(18);
    
    private boolean built = false;
    
    public ColorRenderer()
    {
        Shader.addTask(() -> 
        {
            vao = glGenVertexArrays();
            vbo = glGenBuffers();

            glBindVertexArray(vao);
            glBindBuffer(GL_ARRAY_BUFFER, vbo);

            glEnableVertexAttribArray(0);
            glVertexAttribPointer(0, 2, GL_FLOAT, false, 24, 0);

            glEnableVertexAttribArray(2);  
            glVertexAttribPointer(2, 4, GL_FLOAT, false, 24, 8);
        });
    }
    
    public void setColor(float r, float g, float b, float a)
    {
        this.r = r;
        this.g = g;
        this.b = b;
        this.a = a;
    }
    
    public void addTriangle(float x1, float y1, float x2, float y2, float x3, float y3)
    {
        if(buffer.position() + 18 > buffer.capacity())
        {
            FloatBuffer newBuffer = BufferUtils.createFloatBuffer(buffer.capacity() * 2);
            buffer.flip();
            newBuffer.put(buffer);
            buffer = newBuffer;
        }
        
        buffer.put(x1);
        buffer.put(y1);        
        buffer.put(r);
        buffer.put(g);
        buffer.put(b);
        buffer.put(a);
        
        buffer.put(x2);
        buffer.put(y2);        
        buffer.put(r);
        buffer.put(g);
        buffer.put(b);
        buffer.put(a);
        
        buffer.put(x3);
        buffer.put(y3);        
        buffer.put(r);
        buffer.put(g);
        buffer.put(b);
        buffer.put(a);
    }
    
    public void addRectangle(float minX, float minY, float maxX, float maxY)
    {
        addTriangle(minX, maxY, minX, minY, maxX, maxY);
        addTriangle(maxX, maxY, minX, minY, maxX, minY);
    }
    
    public void build()
    {
        if(!Shader.initDone || buffer.position() == 0)
        {
            throw new ShaderException("build called too early");
        }
        buffer.flip();
        glBindVertexArray(vao);
        glBindBuffer(GL_ARRAY_BUFFER, vbo);
        glBufferData(GL_ARRAY_BUFFER, buffer, GL_STATIC_DRAW);
        built = true;
    }
    
    public void draw()
    {
        if(!built)
        {
            throw new ShaderException("build must be called before drawing");
        }
        glBindVertexArray(vao);
        glBindBuffer(GL_ARRAY_BUFFER, vbo);
        glDrawArrays(GL_TRIANGLES, 0, buffer.limit() / 6);
    }
}
