package me.hammerle.snuviengine.api;

import java.util.Random;
import static org.lwjgl.opengl.GL20.*;

public class Map 
{
    protected final static int PARTION = 16;
    
    private final TextureAtlas textures = new TextureAtlas(Engine.TILE_SIZE, Engine.TILE_SIZE, 16, 16);
    
    private int layers;
    private int width;
    private int height;
    private short[][][] tiles;
    
    private int pWidth;
    private int pHeight;
    private Chunk[][][] chunks = null;
    
    private float cameraX;
    private float cameraY;
    private boolean dirtyCamera = true;
    
    protected Map()
    {
    }
    
    protected void preInit()
    {
    }
    
    protected void postInit()
    {
        textures.generateTexture();
    }
    
    protected void bind(int program)
    {
        
        dirtyCamera = true;
        tick(program);
    }
    
    protected void tick(int program)
    {
        
        if(dirtyCamera)
        {
            dirtyCamera = false;
            glUniform2f(glGetUniformLocation(program, "camera"), cameraX, cameraY);
        }
    }
    
    public TextureAtlas getTextureAtlas()
    {
        return textures;
    }
    
    public void load(String path)
    {
        layers = 2;
        width = 64;
        height = 64;
        tiles = new short[layers][width][height];
        Random r = new Random(100);
        for(int x = 0; x < width; x++)
        {
            for(int y = 0; y < height; y++)
            {
                tiles[0][x][y] = 0;
            }
        }
        for(int x = 0; x < width; x++)
        {
            for(int y = 0; y < height; y++)
            {
                tiles[1][x][y] = (short) (r.nextInt(2));
            }
        }
        
        if(chunks != null)
        {
            for(Chunk[][] a : chunks)
            {
                for(Chunk[] b : a)
                {
                    for(Chunk c : b)
                    {
                        c.delete();
                    }
                }
            }
        }
        
        pWidth = width / PARTION;
        pHeight = height / PARTION;
        
        chunks = new Chunk[layers][pWidth][pHeight];
        for(int l = 0; l < layers; l++)
        {
            for(int x = 0; x < pWidth; x++)
            {
                for(int y = 0; y < pHeight; y++)
                {
                    chunks[l][x][y] = new Chunk();
                }
            }
        }
    }
    
    protected void draw()
    {
        textures.bindTexture();
        for(int l = 0; l < layers; l++)
        {
            for(int x = 0; x < pWidth; x++)
            {
                for(int y = 0; y < pHeight; y++)
                {
                    chunks[l][x][y].draw(tiles[l], x * PARTION, y * PARTION, textures);
                }
            }
        }
    }
    
    public void setCamera(float x, float y)
    {
        cameraX = getViewX(x);
        cameraY = getViewY(y);
        dirtyCamera = true;
    }
    
    public float getCameraX()
    {
        return cameraX;
    }
    
    public float getCameraY()
    {
        return cameraY;
    }
    
    private float getViewX(float x) 
    {
        x -= Engine.WIDTH >> 1;
        if(x < 0)
        {
            return 0;
        }
        float max = width * Engine.TILE_SIZE * Engine.SCALE - Engine.WIDTH;
        if(x > max)
        {
            return max;
        }
        return x;
    }
    
    private float getViewY(float y) 
    {
        y -= Engine.HEIGHT >> 1;
        if(y < 0)
        {
            return 0;
        }
        float max = height * Engine.TILE_SIZE * Engine.SCALE - Engine.HEIGHT;
        if(y > max)
        {
            return max;
        }
        return y;
    }
}
